@@ -11,28 +11,23 @@
 #include "nvk_event.h"
 #include "nvk_mme.h"
 #include "nvk_physical_device.h"
+#include "nvkmd/nvkmd.h"
 
 #include "vk_common_entrypoints.h"
 #include "vk_meta.h"
 #include "vk_pipeline.h"
 
+#include "cl/nvk_query.h"
 #include "compiler/nir/nir.h"
 #include "compiler/nir/nir_builder.h"
-
-#include "nouveau_bo.h"
-#include "nouveau_context.h"
+#include "nvkcl.h"
 
 #include "util/os_time.h"
 
-#include "nvk_cl906f.h"
-#include "nvk_cl9097.h"
-#include "nvk_cla0c0.h"
-#include "nvk_clc597.h"
-
-struct nvk_query_report {
-   uint64_t value;
-   uint64_t timestamp;
-};
+#include "nv_push_cl906f.h"
+#include "nv_push_cl9097.h"
+#include "nv_push_cla0c0.h"
+#include "nv_push_clc597.h"
 
 VKAPI_ATTR VkResult VKAPI_CALL
 nvk_CreateQueryPool(VkDevice device,
@@ -41,7 +36,9 @@ nvk_CreateQueryPool(VkDevice device,
                     VkQueryPool *pQueryPool)
 {
    VK_FROM_HANDLE(nvk_device, dev, device);
+   const struct nvk_physical_device *pdev = nvk_device_physical(dev);
    struct nvk_query_pool *pool;
+   VkResult result;
 
    pool = vk_query_pool_create(&dev->vk, pCreateInfo,
                                pAllocator, sizeof(*pool));
@@ -74,20 +71,20 @@ nvk_CreateQueryPool(VkDevice device,
    pool->query_stride = reports_per_query * sizeof(struct nvk_query_report);
 
    if (pool->vk.query_count > 0) {
-      uint32_t bo_size = pool->query_start +
-                         pool->query_stride * pool->vk.query_count;
-      pool->bo = nouveau_ws_bo_new_mapped(dev->ws_dev, bo_size, 0,
-                                          NOUVEAU_WS_BO_GART |
-                                          NOUVEAU_WS_BO_NO_SHARE,
-                                          NOUVEAU_WS_BO_RDWR,
-                                          &pool->bo_map);
-      if (!pool->bo) {
+      uint32_t mem_size = pool->query_start +
+                          pool->query_stride * pool->vk.query_count;
+      result = nvkmd_dev_alloc_mapped_mem(dev->nvkmd, &dev->vk.base,
+                                          mem_size, 0 /* align_B */,
+                                          NVKMD_MEM_GART,
+                                          NVKMD_MEM_MAP_RDWR,
+                                          &pool->mem);
+      if (result != VK_SUCCESS) {
          vk_query_pool_destroy(&dev->vk, pAllocator, &pool->vk);
-         return vk_error(dev, VK_ERROR_OUT_OF_DEVICE_MEMORY);
+         return result;
       }
 
-      if (dev->ws_dev->debug_flags & NVK_DEBUG_ZERO_MEMORY)
-         memset(pool->bo_map, 0, bo_size);
+      if (pdev->debug_flags & NVK_DEBUG_ZERO_MEMORY)
+         memset(pool->mem->map, 0, mem_size);
    }
 
    *pQueryPool = nvk_query_pool_to_handle(pool);
@@ -106,10 +103,8 @@ nvk_DestroyQueryPool(VkDevice device,
    if (!pool)
       return;
 
-   if (pool->bo) {
-      nouveau_ws_bo_unmap(pool->bo, pool->bo_map);
-      nouveau_ws_bo_destroy(pool->bo);
-   }
+   if (pool->mem)
+      nvkmd_mem_unref(pool->mem);
    vk_query_pool_destroy(&dev->vk, pAllocator, &pool->vk);
 }
 
@@ -117,22 +112,14 @@ static uint64_t
 nvk_query_available_addr(struct nvk_query_pool *pool, uint32_t query)
 {
    assert(query < pool->vk.query_count);
-   return pool->bo->offset + query * sizeof(uint32_t);
-}
-
-static nir_def *
-nvk_nir_available_addr(nir_builder *b, nir_def *pool_addr,
-                       nir_def *query)
-{
-   nir_def *offset = nir_imul_imm(b, query, sizeof(uint32_t));
-   return nir_iadd(b, pool_addr, nir_u2u64(b, offset));
+   return pool->mem->va->addr + query * sizeof(uint32_t);
 }
 
 static uint32_t *
 nvk_query_available_map(struct nvk_query_pool *pool, uint32_t query)
 {
    assert(query < pool->vk.query_count);
-   return (uint32_t *)pool->bo_map + query;
+   return (uint32_t *)pool->mem->map + query;
 }
 
 static uint64_t
@@ -145,23 +132,13 @@ nvk_query_offset(struct nvk_query_pool *pool, uint32_t query)
 static uint64_t
 nvk_query_report_addr(struct nvk_query_pool *pool, uint32_t query)
 {
-   return pool->bo->offset + nvk_query_offset(pool, query);
-}
-
-static nir_def *
-nvk_nir_query_report_addr(nir_builder *b, nir_def *pool_addr,
-                          nir_def *query_start, nir_def *query_stride,
-                          nir_def *query)
-{
-   nir_def *offset =
-      nir_iadd(b, query_start, nir_umul_2x32_64(b, query, query_stride));
-   return nir_iadd(b, pool_addr, offset);
+   return pool->mem->va->addr + nvk_query_offset(pool, query);
 }
 
 static struct nvk_query_report *
 nvk_query_report_map(struct nvk_query_pool *pool, uint32_t query)
 {
-   return (void *)((char *)pool->bo_map + nvk_query_offset(pool, query));
+   return (void *)((char *)pool->mem->map + nvk_query_offset(pool, query));
 }
 
 /**
@@ -474,13 +451,13 @@ nvk_cmd_begin_end_query(struct nvk_cmd_buffer *cmd,
          P_NV9097_SET_REPORT_SEMAPHORE_B(p, report_addr);
          P_NV9097_SET_REPORT_SEMAPHORE_C(p, 0);
          P_NV9097_SET_REPORT_SEMAPHORE_D(p, {
-               .operation = OPERATION_REPORT_ONLY,
-               .pipeline_location = PIPELINE_LOCATION_STREAMING_OUTPUT,
-               .report = xfb_reports[i],
-               .structure_size = STRUCTURE_SIZE_FOUR_WORDS,
-               .sub_report = index,
-               .flush_disable = true,
-               });
+            .operation = OPERATION_REPORT_ONLY,
+            .pipeline_location = PIPELINE_LOCATION_STREAMING_OUTPUT,
+            .report = xfb_reports[i],
+            .structure_size = STRUCTURE_SIZE_FOUR_WORDS,
+            .sub_report = index,
+            .flush_disable = true,
+         });
          report_addr += 2 * sizeof(struct nvk_query_report);
       }
       break;
@@ -717,139 +694,6 @@ load_struct_var(nir_builder *b, nir_variable *var, uint32_t field)
    return nir_load_deref(b, deref);
 }
 
-static void
-nir_write_query_result(nir_builder *b, nir_def *dst_addr,
-                       nir_def *idx, nir_def *flags,
-                       nir_def *result)
-{
-   assert(result->num_components == 1);
-   assert(result->bit_size == 64);
-
-   nir_push_if(b, nir_test_mask(b, flags, VK_QUERY_RESULT_64_BIT));
-   {
-      nir_def *offset = nir_i2i64(b, nir_imul_imm(b, idx, 8));
-      nir_store_global(b, nir_iadd(b, dst_addr, offset), 8, result, 0x1);
-   }
-   nir_push_else(b, NULL);
-   {
-      nir_def *result32 = nir_u2u32(b, result);
-      nir_def *offset = nir_i2i64(b, nir_imul_imm(b, idx, 4));
-      nir_store_global(b, nir_iadd(b, dst_addr, offset), 4, result32, 0x1);
-   }
-   nir_pop_if(b, NULL);
-}
-
-static void
-nir_get_query_delta(nir_builder *b, nir_def *dst_addr,
-                    nir_def *report_addr, nir_def *idx,
-                    nir_def *flags)
-{
-   nir_def *offset =
-      nir_imul_imm(b, idx, 2 * sizeof(struct nvk_query_report));
-   nir_def *begin_addr =
-      nir_iadd(b, report_addr, nir_i2i64(b, offset));
-   nir_def *end_addr =
-      nir_iadd_imm(b, begin_addr, sizeof(struct nvk_query_report));
-
-   /* nvk_query_report::timestamp is the first uint64_t */
-   nir_def *begin = nir_load_global(b, begin_addr, 16, 1, 64);
-   nir_def *end = nir_load_global(b, end_addr, 16, 1, 64);
-
-   nir_def *delta = nir_isub(b, end, begin);
-
-   nir_write_query_result(b, dst_addr, idx, flags, delta);
-}
-
-static void
-nvk_nir_copy_query(nir_builder *b, nir_variable *push, nir_def *i)
-{
-   nir_def *pool_addr = load_struct_var(b, push, 0);
-   nir_def *query_start = nir_u2u64(b, load_struct_var(b, push, 1));
-   nir_def *query_stride = load_struct_var(b, push, 2);
-   nir_def *first_query = load_struct_var(b, push, 3);
-   nir_def *dst_addr = load_struct_var(b, push, 5);
-   nir_def *dst_stride = load_struct_var(b, push, 6);
-   nir_def *flags = load_struct_var(b, push, 7);
-
-   nir_def *query = nir_iadd(b, first_query, i);
-
-   nir_def *avail_addr = nvk_nir_available_addr(b, pool_addr, query);
-   nir_def *available =
-      nir_i2b(b, nir_load_global(b, avail_addr, 4, 1, 32));
-
-   nir_def *partial = nir_test_mask(b, flags, VK_QUERY_RESULT_PARTIAL_BIT);
-   nir_def *write_results = nir_ior(b, available, partial);
-
-   nir_def *report_addr =
-      nvk_nir_query_report_addr(b, pool_addr, query_start, query_stride,
-                                query);
-   nir_def *dst_offset = nir_imul(b, nir_u2u64(b, i), dst_stride);
-
-   /* Timestamp queries are the only ones use a single report */
-   nir_def *is_timestamp =
-      nir_ieq_imm(b, query_stride, sizeof(struct nvk_query_report));
-
-   nir_def *one = nir_imm_int(b, 1);
-   nir_def *num_reports;
-   nir_push_if(b, is_timestamp);
-   {
-      nir_push_if(b, write_results);
-      {
-         /* This is the timestamp case.  We add 8 because we're loading
-          * nvk_query_report::timestamp.
-          */
-         nir_def *timestamp =
-            nir_load_global(b, nir_iadd_imm(b, report_addr, 8), 8, 1, 64);
-
-         nir_write_query_result(b, nir_iadd(b, dst_addr, dst_offset),
-                                nir_imm_int(b, 0), flags, timestamp);
-      }
-      nir_pop_if(b, NULL);
-   }
-   nir_push_else(b, NULL);
-   {
-      /* Everything that isn't a timestamp has the invariant that the
-       * number of destination entries is equal to the query stride divided
-       * by the size of two reports.
-       */
-      num_reports = nir_udiv_imm(b, query_stride,
-                                 2 * sizeof(struct nvk_query_report));
-
-      nir_push_if(b, write_results);
-      {
-         nir_variable *r =
-            nir_local_variable_create(b->impl, glsl_uint_type(), "r");
-         nir_store_var(b, r, nir_imm_int(b, 0), 0x1);
-
-         nir_push_loop(b);
-         {
-            nir_push_if(b, nir_ige(b, nir_load_var(b, r), num_reports));
-            {
-               nir_jump(b, nir_jump_break);
-            }
-            nir_pop_if(b, NULL);
-
-            nir_get_query_delta(b, nir_iadd(b, dst_addr, dst_offset),
-                                report_addr, nir_load_var(b, r), flags);
-
-            nir_store_var(b, r, nir_iadd_imm(b, nir_load_var(b, r), 1), 0x1);
-         }
-         nir_pop_loop(b, NULL);
-      }
-      nir_pop_if(b, NULL);
-   }
-   nir_pop_if(b, NULL);
-
-   num_reports = nir_if_phi(b, one, num_reports);
-
-   nir_push_if(b, nir_test_mask(b, flags, VK_QUERY_RESULT_WITH_AVAILABILITY_BIT));
-   {
-      nir_write_query_result(b, nir_iadd(b, dst_addr, dst_offset),
-                             num_reports, flags, nir_b2i64(b, available));
-   }
-   nir_pop_if(b, NULL);
-}
-
 static nir_shader *
 build_copy_queries_shader(void)
 {
@@ -875,57 +719,50 @@ build_copy_queries_shader(void)
    nir_variable *push = nir_variable_create(b->shader, nir_var_mem_push_const,
                                             push_iface_type, "push");
 
-   nir_def *query_count = load_struct_var(b, push, 4);
+   b->shader->info.workgroup_size[0] = 32;
 
-   nir_variable *i = nir_local_variable_create(b->impl, glsl_uint_type(), "i");
-   nir_store_var(b, i, nir_imm_int(b, 0), 0x1);
-
-   nir_push_loop(b);
-   {
-      nir_push_if(b, nir_ige(b, nir_load_var(b, i), query_count));
-      {
-         nir_jump(b, nir_jump_break);
-      }
-      nir_pop_if(b, NULL);
-
-      nvk_nir_copy_query(b, push, nir_load_var(b, i));
-
-      nir_store_var(b, i, nir_iadd_imm(b, nir_load_var(b, i), 1), 0x1);
-   }
-   nir_pop_loop(b, NULL);
+   nvk_copy_queries(b, load_struct_var(b, push, 0), load_struct_var(b, push, 1),
+                    load_struct_var(b, push, 2), load_struct_var(b, push, 3),
+                    load_struct_var(b, push, 4), load_struct_var(b, push, 5),
+                    load_struct_var(b, push, 6), load_struct_var(b, push, 7));
 
    return build.shader;
 }
 
-static VkResult
-get_copy_queries_pipeline(struct nvk_device *dev,
-                          VkPipelineLayout layout,
-                          VkPipeline *pipeline_out)
+static struct nvk_shader *
+atomic_set_or_destroy_shader(struct nvk_device *dev,
+                             struct nvk_shader **shader_ptr,
+                             struct nvk_shader *shader,
+                             const VkAllocationCallbacks *alloc)
 {
-   const char key[] = "nvk-meta-copy-query-pool-results";
-   VkPipeline cached = vk_meta_lookup_pipeline(&dev->meta, key, sizeof(key));
-   if (cached != VK_NULL_HANDLE) {
-      *pipeline_out = cached;
+   struct nvk_shader *old_shader = p_atomic_cmpxchg(shader_ptr, NULL, shader);
+   if (old_shader == NULL) {
+      return shader;
+   } else {
+      vk_shader_destroy(&dev->vk, &shader->vk, alloc);
+      return old_shader;
+   }
+}
+
+static VkResult
+get_copy_queries_shader(struct nvk_device *dev,
+                        struct nvk_shader **shader_out)
+{
+   struct nvk_shader *shader = p_atomic_read(&dev->copy_queries);
+   if (shader != NULL) {
+      *shader_out = shader;
       return VK_SUCCESS;
    }
 
-   const VkPipelineShaderStageNirCreateInfoMESA nir_info = {
-      .sType = VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_NIR_CREATE_INFO_MESA,
-      .nir = build_copy_queries_shader(),
-   };
-   const VkComputePipelineCreateInfo info = {
-      .sType = VK_STRUCTURE_TYPE_COMPUTE_PIPELINE_CREATE_INFO,
-      .stage = {
-         .sType = VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO,
-         .pNext = &nir_info,
-         .stage = VK_SHADER_STAGE_COMPUTE_BIT,
-         .pName = "main",
-      },
-      .layout = layout,
-   };
+   nir_shader *nir = build_copy_queries_shader();
+   VkResult result = nvk_compile_nir_shader(dev, nir, &dev->vk.alloc, &shader);
+   if (result != VK_SUCCESS)
+      return result;
 
-   return vk_meta_create_compute_pipeline(&dev->vk, &dev->meta, &info,
-                                          key, sizeof(key), pipeline_out);
+   *shader_out = atomic_set_or_destroy_shader(dev, &dev->copy_queries,
+                                              shader, &dev->vk.alloc);
+
+   return VK_SUCCESS;
 }
 
 static void
@@ -938,11 +775,16 @@ nvk_meta_copy_query_pool_results(struct nvk_cmd_buffer *cmd,
                                  VkQueryResultFlags flags)
 {
    struct nvk_device *dev = nvk_cmd_buffer_device(cmd);
-   struct nvk_descriptor_state *desc = &cmd->state.cs.descriptors;
-   VkResult result;
+
+   struct nvk_shader *shader;
+   VkResult result = get_copy_queries_shader(dev, &shader);
+   if (result != VK_SUCCESS) {
+      vk_command_buffer_set_error(&cmd->vk, result);
+      return;
+   }
 
    const struct nvk_copy_query_push push = {
-      .pool_addr = pool->bo->offset,
+      .pool_addr = pool->mem->va->addr,
       .query_start = pool->query_start,
       .query_stride = pool->query_stride,
       .first_query = first_query,
@@ -951,178 +793,8 @@ nvk_meta_copy_query_pool_results(struct nvk_cmd_buffer *cmd,
       .dst_stride = dst_stride,
       .flags = flags,
    };
-
-   const char key[] = "nvk-meta-copy-query-pool-results";
-   const VkPushConstantRange push_range = {
-      .stageFlags = VK_SHADER_STAGE_COMPUTE_BIT,
-      .size = sizeof(push),
-   };
-   VkPipelineLayout layout;
-   result = vk_meta_get_pipeline_layout(&dev->vk, &dev->meta, NULL, &push_range,
-                                        key, sizeof(key), &layout);
-   if (result != VK_SUCCESS) {
-      vk_command_buffer_set_error(&cmd->vk, result);
-      return;
-   }
-
-   VkPipeline pipeline;
-   result = get_copy_queries_pipeline(dev, layout, &pipeline);
-   if (result != VK_SUCCESS) {
-      vk_command_buffer_set_error(&cmd->vk, result);
-      return;
-   }
-
-   /* Save pipeline and push constants */
-   struct nvk_shader *shader_save = cmd->state.cs.shader;
-   uint8_t push_save[NVK_MAX_PUSH_SIZE];
-   memcpy(push_save, desc->root.push, NVK_MAX_PUSH_SIZE);
-
-   dev->vk.dispatch_table.CmdBindPipeline(nvk_cmd_buffer_to_handle(cmd),
-                                          VK_PIPELINE_BIND_POINT_COMPUTE,
-                                          pipeline);
-
-   vk_common_CmdPushConstants(nvk_cmd_buffer_to_handle(cmd), layout,
-                              VK_SHADER_STAGE_COMPUTE_BIT, 0, sizeof(push), &push);
-
-   nvk_CmdDispatchBase(nvk_cmd_buffer_to_handle(cmd), 0, 0, 0, 1, 1, 1);
-
-   /* Restore pipeline and push constants */
-   if (shader_save)
-      nvk_cmd_bind_compute_shader(cmd, shader_save);
-   memcpy(desc->root.push, push_save, NVK_MAX_PUSH_SIZE);
-}
-
-void
-nvk_mme_copy_queries(struct mme_builder *b)
-{
-   if (b->devinfo->cls_eng3d < TURING_A)
-      return;
-
-   struct mme_value64 dst_addr = mme_load_addr64(b);
-   struct mme_value64 dst_stride = mme_load_addr64(b);
-   struct mme_value64 avail_addr = mme_load_addr64(b);
-   struct mme_value64 report_addr = mme_load_addr64(b);
-
-   struct mme_value query_count = mme_load(b);
-   struct mme_value control = mme_load(b);
-
-   struct mme_value flags = control;
-   struct mme_value write64 =
-      mme_and(b, flags, mme_imm(VK_QUERY_RESULT_64_BIT));
-   struct mme_value query_stride =
-      mme_merge(b, mme_zero(), control, 0, 16, 8);
-   struct mme_value is_timestamp =
-      mme_merge(b, mme_zero(), control, 0, 1, 24);
-
-   mme_while(b, ugt, query_count, mme_zero()) {
-      struct mme_value dw_per_query = mme_srl(b, query_stride, mme_imm(2));
-      mme_tu104_read_fifoed(b, report_addr, dw_per_query);
-      mme_free_reg(b, dw_per_query);
-
-      struct mme_value64 write_addr = mme_mov64(b, dst_addr);
-      struct mme_value report_count = mme_srl(b, query_stride, mme_imm(4));
-      mme_while(b, ugt, report_count, mme_zero()) {
-         struct mme_value result_lo = mme_alloc_reg(b);
-         struct mme_value result_hi = mme_alloc_reg(b);
-         struct mme_value64 result = mme_value64(result_lo, result_hi);
-
-         mme_if(b, ine, is_timestamp, mme_zero()) {
-            mme_load_to(b, mme_zero());
-            mme_load_to(b, mme_zero());
-            mme_load_to(b, result_lo);
-            mme_load_to(b, result_hi);
-            mme_sub_to(b, report_count, report_count, mme_imm(1));
-         }
-         mme_if(b, ieq, is_timestamp, mme_zero()) {
-            struct mme_value begin_lo = mme_load(b);
-            struct mme_value begin_hi = mme_load(b);
-            struct mme_value64 begin = mme_value64(begin_lo, begin_hi);
-            mme_load_to(b, mme_zero());
-            mme_load_to(b, mme_zero());
-
-            struct mme_value end_lo = mme_load(b);
-            struct mme_value end_hi = mme_load(b);
-            struct mme_value64 end = mme_value64(end_lo, end_hi);
-            mme_load_to(b, mme_zero());
-            mme_load_to(b, mme_zero());
-
-            mme_sub64_to(b, result, end, begin);
-            mme_sub_to(b, report_count, report_count, mme_imm(2));
-
-            mme_free_reg64(b, begin);
-            mme_free_reg64(b, end);
-         }
-
-         mme_store_global(b, write_addr, result_lo);
-         mme_add64_to(b, write_addr, write_addr, mme_imm64(4));
-         mme_if(b, ine, write64, mme_zero()) {
-            mme_store_global(b, write_addr, result_hi);
-            mme_add64_to(b, write_addr, write_addr, mme_imm64(4));
-         }
-      }
-
-      struct mme_value with_availability =
-         mme_and(b, flags, mme_imm(VK_QUERY_RESULT_WITH_AVAILABILITY_BIT));
-      mme_if(b, ine, with_availability, mme_zero()) {
-         mme_tu104_read_fifoed(b, avail_addr, mme_imm(1));
-         struct mme_value avail = mme_load(b);
-         mme_store_global(b, write_addr, avail);
-         mme_if(b, ine, write64, mme_zero()) {
-            mme_add64_to(b, write_addr, write_addr, mme_imm64(4));
-            mme_store_global(b, write_addr, mme_zero());
-         }
-      }
-      mme_free_reg(b, with_availability);
-
-      mme_add64_to(b, avail_addr, avail_addr, mme_imm64(4));
-
-      mme_add64_to(b, report_addr, report_addr,
-                   mme_value64(query_stride, mme_zero()));
-
-      mme_add64_to(b, dst_addr, dst_addr, dst_stride);
-
-      mme_sub_to(b, query_count, query_count, mme_imm(1));
-   }
-}
-
-static void
-nvk_cmd_copy_query_pool_results_mme(struct nvk_cmd_buffer *cmd,
-                                    struct nvk_query_pool *pool,
-                                    uint32_t first_query,
-                                    uint32_t query_count,
-                                    uint64_t dst_addr,
-                                    uint64_t dst_stride,
-                                    VkQueryResultFlags flags)
-{
-   /* TODO: vkCmdCopyQueryPoolResults() with a compute shader */
-   ASSERTED struct nvk_device *dev = nvk_cmd_buffer_device(cmd);
-   assert(nvk_device_physical(dev)->info.cls_eng3d >= TURING_A);
-
-   struct nv_push *p = nvk_cmd_buffer_push(cmd, 13);
-   P_IMMD(p, NVC597, SET_MME_DATA_FIFO_CONFIG, FIFO_SIZE_SIZE_4KB);
-   P_1INC(p, NVC597, CALL_MME_MACRO(NVK_MME_COPY_QUERIES));
-
-   P_INLINE_DATA(p, dst_addr >> 32);
-   P_INLINE_DATA(p, dst_addr);
-   P_INLINE_DATA(p, dst_stride >> 32);
-   P_INLINE_DATA(p, dst_stride);
-
-   uint64_t avail_start = nvk_query_available_addr(pool, first_query);
-   P_INLINE_DATA(p, avail_start >> 32);
-   P_INLINE_DATA(p, avail_start);
-
-   uint64_t report_start = nvk_query_report_addr(pool, first_query);
-   P_INLINE_DATA(p, report_start >> 32);
-   P_INLINE_DATA(p, report_start);
-
-   P_INLINE_DATA(p, query_count);
-
-   uint32_t is_timestamp = pool->vk.query_type == VK_QUERY_TYPE_TIMESTAMP;
-
-   uint32_t control = (flags & 0xff) |
-                      (pool->query_stride << 8) |
-                      (is_timestamp << 24);
-   P_INLINE_DATA(p, control);
+   nvk_cmd_dispatch_shader(cmd, shader, &push, sizeof(push),
+                           DIV_ROUND_UP(query_count, 32), 1, 1);
 }
 
 VKAPI_ATTR void VKAPI_CALL
@@ -1156,7 +828,8 @@ nvk_CmdCopyQueryPoolResults(VkCommandBuffer commandBuffer,
       }
    }
 
-   uint64_t dst_addr = nvk_buffer_address(dst_buffer, dstOffset);
+   uint64_t dst_addr = vk_buffer_address(&dst_buffer->vk, dstOffset);
    nvk_meta_copy_query_pool_results(cmd, pool, firstQuery, queryCount,
                                     dst_addr, stride, flags);
 }
+
